-- DIM 2.0.applescript
-- DIM 2.0

--  Created by G.J. Parker on 2005/10/29.
--  Copyright 2005. All rights reserved.

(* this applescript studio application records the positions of the icons on the desktop along w/ the screen resolution in the plist file
~/Library/Preferences/com.parker9.DIM_2.0.plist. it also allows the user to have the code automatically restore the icon positions and quit. 
to get back to memorizing and/or selecting the auto-restore, the user needs to drag and drop anything onto the application.
finally, if the screen resolution is different, we simply place the icons in the same relative positions as initially set *)

(* set properties *)
property IconsAll : {} (* list of all icons on desktop *)
property PositionAll : {} (* corresponding list of positions of the icons on desktop *)
property ScreenSize : {} (* screem size for the previous lists *)
property prompt_p : true (* should user see main window, or default to just 'Restore' and quit? *)
property start_p : true (* true when first started, but quickly false *)

on open names
	set start_p to false (* things were dropped on us, so don't just restore and quit! *)
end open

on activated theObject
	if prompt_p = false and start_p = true then (* if restore and quit and we just started w/o any drag and drop, then restore and quit! *)
		restore() of me
		quit
	else
		tell application "Finder" to set x to count of (items of desktop)
		tell window "DIM 2.0" to set contents of text field "num_current" to ("Number of current Desktop Icons: " & x as string)
	end if
	set start_p to false (* we just didn't just start or we did, but we had drag and dropped on us *)
end activated

on clicked theObject
	if name of theObject is "restore" then (* user wants to restore positions *)
		restore() of me
	else if name of theObject is "memorize" then (* user wants to memorize new positions *)
		memorize() of me
	else if name of theObject is "auto_restore" then (* user is toggling the flag *)
		tell window "DIM 2.0"
			set a to state of button "auto_restore"
			if state of button "auto_restore" is 1 then
				set prompt_p to false
				set visible of text field "hint" to true
			else
				set prompt_p to true
				set visible of text field "hint" to false
			end if
		end tell
		set contents of default entry "prompt_p" of user defaults to prompt_p (* and store it so it's persisent *)
	end if
	tell application "Finder" to set x to count of (items of desktop)
	tell window "DIM 2.0" to set contents of text field "num_current" to ("Number of current Desktop Icons: " & x as string)
end clicked

on should quit after last window closed theObject -- quit when the window is closed
	quit
end should quit after last window closed

on will finish launching theObject
	(* we have to check that they are running tiger, otherwise the script won't work! *)
	if (system attribute "sysv") < 4160 then (* they need Tiger- 4160 converted to hex is 1040 = 10.4.0 *)
		set request to display dialog "Desktop Icon Manager 2.0 only works for Mac OS X 10.4 and later" buttons {"Quit"} default button "Quit"
		quit
	else
		
		(* there's a difference in applescript and applescript studio- properties are not persisent!
	  so, you store these things in a preference file... *)
		make new default entry at end of default entries of user defaults with properties {name:"IconsAll", contents:{}} (* doesn't destroy values if they exist *)
		make new default entry at end of default entries of user defaults with properties {name:"PositionAll", contents:{}}
		make new default entry at end of default entries of user defaults with properties {name:"ScreenSize", contents:{}}
		make new default entry at end of default entries of user defaults with properties {name:"prompt_p", contents:true}
		set IconsAll to contents of default entry "IconsAll" of user defaults (* load up values *)
		set PositionAll to contents of default entry "PositionAll" of user defaults
		set ScreenSize to contents of default entry "ScreenSize" of user defaults
		set prompt_p to contents of default entry "prompt_p" of user defaults
		
		tell window "DIM 2.0" (* set up window elements *)
			if (count of IconsAll) = 0 then
				memorize() of me
			end if
			set visible of progress indicator "spinning" to false
			set visible of text field "message" to false
			if prompt_p = true then
				set state of button "auto_restore" to 0
				set visible of text field "hint" to false
			else
				set state of button "auto_restore" to 1
				set visible of text field "hint" to true
			end if
			set contents of text field "num_memorized" to ("Number of memorized Desktop Icons: " & (count of IconsAll) as string)
			tell application "Finder" to set x to count of (items of desktop)
			set contents of text field "num_current" to ("Number of current Desktop Icons: " & x as string)
			set visible of text field "num_memorized" to true
			set visible of text field "num_current" to true
		end tell
	end if
end will finish launching

on choose menu item theObject
	if (name of theObject) is "close" then
		quit
	end if
end choose menu item

on restore() -- restore icon positions
	tell window "DIM 2.0" -- setup up window
		set enabled of button "memorize" to false
		set enabled of button "restore" to false
		set enabled of button "auto_restore" to false
		set visible of text field "num_memorized" to false
		set visible of text field "num_current" to false
		set contents of text field "message" to "Restoring Icon Positions..."
		set visible of text field "message" to true
		set visible of progress indicator "spinning" to true
		start progress indicator "spinning"
	end tell
	tell application "Finder" -- now onto the actual restore positions
		copy the bounds of the desktop's window to the NewScreenSize -- get current resolution
		set aSet to arrangement of (icon view options of desktop's window)
		if aSet  not arranged then
			set arrangement of (icon view options of desktop's window) to not arranged -- we have to turn off Snap to Grid or whatever else
		end if
		
		set newdx to item 1 of NewScreenSize -- scaling parameters
		set newdy to item 2 of NewScreenSize
		set xrel to ((item 3 of NewScreenSize) - newdx) / ((item 3 of ScreenSize) - (item 1 of ScreenSize))
		set yrel to ((item 4 of NewScreenSize) - newdy) / ((item 4 of ScreenSize) - (item 2 of ScreenSize))
		repeat with i from 1 to number of items of IconsAll -- for each desktop icon
			ignoring application responses
				tell application "Finder" to set desktop position of item (item i of IconsAll) of desktop to 
					{newdx + (0.5 + ((item 1 of (item i of PositionAll)) - newdx) * xrel) div 1, 
						newdy + (0.5 + ((item 2 of (item i of PositionAll)) - newdy) * yrel) div 1} -- new position
				
			end ignoring
		end repeat
	end tell
	tell window "DIM 2.0" -- reset window
		set enabled of button "memorize" to true
		set enabled of button "restore" to true
		set enabled of button "auto_restore" to true
		set visible of text field "message" to false
		set visible of progress indicator "spinning" to false
		stop progress indicator "spinning"
		tell application "Finder" to set x to count of (items of desktop)
		set contents of text field "num_current" to ("Number of current Desktop Icons: " & x as string)
		set visible of text field "num_memorized" to true
		set visible of text field "num_current" to true
	end tell
end restore

on memorize() -- memorize icon positions
	tell window "DIM 2.0" -- set up window
		set enabled of button "memorize" to false
		set enabled of button "restore" to false
		set enabled of button "auto_restore" to false
		set visible of text field "num_memorized" to false
		set visible of text field "num_current" to false
		set contents of text field "message" to "Memorizing Icon Positions..."
		set visible of text field "message" to true
		set visible of progress indicator "spinning" to true
		start progress indicator "spinning"
	end tell
	copy {} to IconsAll -- zero out everything
	copy {} to PositionAll
	copy {} to ScreenSize
	tell application "Finder" -- go get names, positions and current screen size
		set IconsAll to name of items of desktop
		set PositionAll to desktop position of items of desktop
		copy the bounds of the desktop's window to the ScreenSize
	end tell
	set contents of default entry "IconsAll" of user defaults to IconsAll --store them, damn applescript studio
	set contents of default entry "PositionAll" of user defaults to PositionAll
	set contents of default entry "ScreenSize" of user defaults to ScreenSize
	
	tell window "DIM 2.0" --reset window
		set enabled of button "memorize" to true
		set enabled of button "restore" to true
		set enabled of button "auto_restore" to true
		set visible of text field "message" to false
		set visible of progress indicator "spinning" to false
		stop progress indicator "spinning"
		set contents of text field "num_memorized" to ("Number of memorized Desktop Icons: " & (count of IconsAll) as string)
		set contents of text field "num_current" to ("Number of current Desktop Icons: " & (count of IconsAll) as string)
		set visible of text field "num_memorized" to true
		set visible of text field "num_current" to true
	end tell
end memorize